/**
 * \file   seconsole.c
 * \author Piotr Koenig
 * \author Sander Elektronik AG
 * \date   2019.12.19
 * \copyright (C) 2019 Sander Elektronik AG. All rights reserved.
 * \brief  Console interface to host processor
 *
 */
/* ===== Revision History: ================================================
Date       Ver   Who Why
---------- ----- --- ------------------------------------------------------
2019.12.18 1.00  PK  Start conversion for nrf52
2015.06.26 1.00  PK  Start
===========================================================================*/
#include <string.h>
#include <stdio.h>
#include <ctype.h>
#include <stdlib.h>
//#include "nrf.h"
//#include "nrf_temp.h"
//#include "nrf_adc.h"
#include "seconsole.h"

#define true 1
#define false 0
uint32_t nrf_mesh_serial_tx(uint8_t* p_data, uint32_t length);

static const char strCls[] = "\x1b[2J\x1b[H";	// [2J is clear screen, H - cursor home
uint8_t hexTab[] = "0123456789ABCDEF";

const char buildDate[] = __DATE__;
const char buildTime[] = __TIME__;

#ifdef BOARD_PCA10040
const char ifType[] = "nRF52832";
#elif defined BOARD_PCA10056
const char ifType[] = "nRF52840";
#elif defined BOARD_CUSTOM
const char ifType[] = "OP-RADIO2";
#else
#error "Board not defined"
#endif

const char *textInfo[] = {
 ifType
,"(c)2020 SanderElektronik AG"
,buildDate
,buildTime
};

//#define PROG_WIDTH 106
//int16_t progIndex, progDir, progDelay, progDelayVal;

#ifdef PROCESS_ESCAPES
int16_t escapeFlag;
#endif //ifdef PROCESS_ESCAPES
int16_t lastPoll;
//char outStr[OUT_STR_SIZE];
//uint16_t memBuf[0x100];
uint8_t rstState;
uint8_t debugFlag;
uint8_t quietFlag, noTimeFlag;
//volatile int32_t adc_sample, adc_count;
//STDIO radio_stdio;
TCONN uartTc;
//TCONN radioTc;
volatile uint32_t srmMsecs, srmMsecsCurr, srmMsecsLast;
volatile uint16_t srmTimer1;
volatile uint8_t srmTimer2;
volatile uint8_t srmTimer3;
extern uint8_t tCmdsSize;
extern TCMDS tCmds[];

uint16_t myAddr;
uint8_t	myNwId;
uint8_t myType;
uint32_t myMac;
char echoInput;

char *getProgRev(char *str) {
    if (str == 0)
      return "";
    strcpy (str, "rev. ");
    strcat (str, APP_VERSION);
    return str;
}

char *getProgTitle(char *str) {
    if (str == 0)
      return "";
        //str = outStr;
    strcpy (str, ifType);
#ifdef DEBUG
    strcat (str, "dbg");
#elif defined NDEBUG
    strcat (str, "rel");
#else //defined NDEBUG
    strcat (str, "???");
#endif //ifdef DEBUG
    strcat (str, ", v. ");
    strcat (str, APP_VERSION);
    strcat (str, ", ");
    strcat (str, buildDate);
    strcat (str, ", ");
    strcat (str, buildTime);
    strcat (str, ", ");
    strcat (str, COMPILER_VERSION);
    return str;
}

unsigned dbgLen;
void processVersion(pTCONN ptc) {
  getProgTitle(ptc->outStr);
  dbgLen = strlen(ptc->outStr);
  put_str(ptc, ptc->outStr);
  put_crlf (ptc);
  //debugFlag |= 0x10;
}

void processRevision(pTCONN ptc) {
  //ptc->outStr[0] = hexTab[(PROG_VERSION >> 8) & 0xf];
  //ptc->outStr[1] = '.';
  //ptc->outStr[2] = hexTab[(PROG_VERSION >> 4) & 0xf];
  //ptc->outStr[3] = hexTab[(PROG_VERSION >> 0) & 0xf];
  //ptc->outStr[4] = '\n';
  //ptc->outStr[5] = '\0';
  //sprintf (ptc->outStr, "%x.%02x\n", (PROG_VERSION >> 8), (PROG_VERSION & 0xff));
  //put_str(ptc, ptc->outStr);
  getProgRev(ptc->outStr);
  dbgLen = strlen(ptc->outStr);
  put_str(ptc, ptc->outStr);
  put_crlf (ptc);
  
}

//void processRng(pTCONN ptc) {
//	sprintf (outStr, "RNG: 0x%X\n", randomDelayCounter);
//	put_outstr(ptc);
//}

#ifndef NRF_APP_PRIORITY_HIGH
#define NRF_APP_PRIORITY_HIGH 1
#endif

void initConsole(void) {
  debugFlag = 0;
  echoInput = 0;
#ifdef PROCESS_ESCAPES
  escapeFlag = 0;
#endif //ifdef PROCESS_ESCAPES
  quietFlag = 0;
  noTimeFlag = 0;
  //adc_config();
  memset (&uartTc, 0, sizeof (uartTc));
  uartTc.tabCmdSel = -1;
  uartTc.outSize = sizeof (uartTc.outStr);
  //memset (&radioTc, 0, sizeof (radioTc));
  //radioTc.tabCmdSel = -1;
  //radioTc.outSize = sizeof (radioTc.outStr);
  //radioTc.devType = 1;
}

char *skipChars (char *ptr) {
  while (*ptr != 0 && *ptr != ' ')
    ptr++;
  return ptr;
}

char *skipSpaces (char *ptr) {
  while (*ptr != 0 && *ptr == ' ')
    ptr++;
  return ptr;
}

char *skipWord (char *ptr) {
  ptr = skipChars(ptr);
  ptr = skipSpaces(ptr);
  return ptr;
}

uint32_t seatoi (char *ptr) {
  int8_t digit;
  uint32_t val = 0;
  if (*ptr) {
    if (ptr[0] == '0' && ptr[1] == 'x') {
      ptr += 2;
      while (*ptr) {
        if (*ptr >= '0' && *ptr <= '9')
          digit = *ptr - '0';
        else if (*ptr >= 'a' && *ptr <= 'f')
          digit = *ptr - 'a' + 10;
        else if (*ptr >= 'A' && *ptr <= 'F')
          digit = *ptr - 'A' + 10;
        else 
          break;
        val <<= 4;
        val += digit;
        ptr ++;
      }
    } else {
      while (*ptr) {
        if (*ptr < '0' || *ptr > '9')
          break;
        digit = *ptr - '0';
        val *= 10;
        val += digit;
        ptr ++;
      }
    }
  }
  return val;
}

void processDebug (pTCONN ptc) {
  char *ptr = skipWord (ptc->cmdStr);
  if (*ptr)
    debugFlag = seatoi(ptr);
  put_str(ptc, "Debug flag: ");
  showByteHex(ptc, debugFlag);
  put_crlf(ptc);
}

int8_t getParameterOnOff (char *ptr) {
    int8_t val = -1;
    ptr = skipWord (ptr);
    if (*ptr) {
        if (*ptr >= '0' && *ptr <= '9')
            val = *ptr - '0';
        else if (strncasecmp ((const char *)ptr, "off", 3) == 0)
            val = 0;
        else if (strncasecmp ((const char *)ptr, "on", 2) == 0)
            val = 1;
    }
    return val;
}

void processLed (pTCONN ptc) {
  int8_t val;
  unsigned mask;
  char *ptr = skipWord (ptc->cmdStr);
  if (*ptr != '\0') {
    int8_t led = *ptr - '0';
#ifdef TEST_LEDS_ON_JP1
    if (led < 0 || led > 3) {
      put_str(ptc, "Invalid led index (0..3).\n");
      return;
    }
#else //ifdef TEST_LEDS_ON_JP1
    switch (led) {
    case 0: mask = BSP_LED_0_MASK; break;
    case 1: mask = BSP_LED_1_MASK; break;
    case 2: mask = BSP_LED_2_MASK; break;
    case 3: mask = BSP_LED_3_MASK; break;
    default:
      put_str(ptc, "Invalid led index (0..3).\n");
      return;
    }
#endif //else ifdef TEST_LEDS_ON_JP1
    val = getParameterOnOff (ptr);
    switch (val) {
    case 0: LEDS_OFF(mask); break;
    case 1: LEDS_ON(mask); break;
    case 2: LEDS_INVERT(mask); break;
    default: break;
    }
    val = LED_IS_ON(mask) ? 1 : 0;
    put_str(ptc, "LED");
    put_ch(ptc, led + '0');
    put_str(ptc, " is ");
    put_ch(ptc, val + '0');
    put_crlf(ptc);
  }
}

void processCls (pTCONN ptc) {
    put_str(ptc, (char *)strCls);
}

#ifdef REDIRECT_STDOUT
void processStdout (pTCONN ptc) {
  int16_t val = getParameterOnOff (ptc->cmdStr);
  if (val == 0 || val == 1)
    ptc->printfEnable = val;
  put_str(ptc, "stdout is");
  put_ch(ptc, ptc->printfEnable + '0');
  put_crlf(ptc);
}
#endif // ifdef REDIRECT_STDOUT

volatile int16_t delayValue;
void tcDelay (int16_t delay) {
  while (delay)
    delayValue = delay--;
}

void showAddress (pTCONN ptc) {
    put_str (ptc, "addr: ");
    put_str (ptc, utoa(myNwId, ptc->outStr, 10));
    put_ch (ptc, ':');
    put_str (ptc, utoa(myAddr, ptc->outStr, 10));
    put_str (ptc, ", type: ");
    put_str (ptc, utoa(myType, ptc->outStr, 10));
    put_ch (ptc, '\n');
}

void showMac (pTCONN ptc) {
    put_str (ptc, "mac: 0x");
    put_str (ptc, ltoa(myMac, ptc->outStr, 16));
    put_ch (ptc, '\n');
}

/* temporary address overwrite */
void processAddress (pTCONN ptc) {
  char *ptr = skipWord (ptc->cmdStr);
  if (*ptr) {
    uint16_t val = seatoi(ptr);
    if (val && val != myAddr)
      myAddr = val;
  }
  showAddress (ptc);
}

void processMac (pTCONN ptc) {
  char *ptr = skipWord (ptc->cmdStr);
  if (*ptr) {
    uint32_t val = seatoi(ptr);
    if (val != myMac) {
      myMac = val;
      myAddr = (uint16_t)(myMac & 0xffff);
      //eepromWriteDWord (EE_MY_MAC, myMac);
    }
  }
  showMac (ptc);
}

void processNwid (pTCONN ptc) {
  char *ptr = skipWord (ptc->cmdStr);
  if (*ptr) {
    uint8_t val = seatoi(ptr);
    if (val != myNwId) {
      myNwId = val;
      //eepromWriteByte (EE_NW_ID, myNwId);
    }
  }
  showAddress (ptc);
}

void processType (pTCONN ptc) {
  char *ptr = skipWord (ptc->cmdStr);
  if (*ptr) {
    if (*ptr == '?') {
      put_str(ptc, "0-ELU, 1-GWY, 2-REL\n");
    } else {
      uint8_t val = seatoi(ptr);
      if (val != myType) {
        myType = val;
        //eepromWriteByte (EE_TYPE, myType);
      }
    }
  }
  put_str (ptc, utoa(myType, ptc->outStr, 10));
  put_ch (ptc, '\n');
}

//void getTelnetOutput(pTCONN ptc) {
//    if (ptc->printfEnable && ptc->tcpOutPtr) {
//        put_array(ptc, ptc->stdoutBuf, ptc->tcpOutPtr);
//        ptc->tcpOutPtr = 0;
//    }
//}

char isoctdig(char c) {
  if (c < '0')
    return 0;
  if (c > '7')
    return 0;
  return 1;
}

char *convertControlChars(char *ptr) {
  int16_t src = 0, dst = 0;
  while (ptr[src]) {
    if (ptr[src] == '\\') {
      if (isoctdig(ptr[src + 1]) && isoctdig(ptr[src + 2]) && isoctdig(ptr[src + 3])) {
        ptr[dst++] = (ptr[src + 1] - '0') * 8 * 8 + (ptr[src + 2] - '0') * 8 + (ptr[src + 3] - '0');
        src += 3;
      } else {
        switch (ptr[src + 1]) {
          case ' ': ptr[dst++] = ' '; src += 2; break;
          case 'n': ptr[dst++] = '\n'; src += 2; break;
          case 'r': ptr[dst++] = '\r'; src += 2; break;
          case 'b': ptr[dst++] = '\b'; src += 2; break;
          case 't': ptr[dst++] = '\t'; src += 2; break;
          case '\\': ptr[dst++] = '\\'; src += 2; break;
          default: ptr[dst++] = '\\'; src++; break;
        }
      }
    } else {
      ptr[dst++] = ptr[src++];
    }
  }
  ptr[dst++] = 0;
  return ptr;
}

void processStatus (pTCONN ptc) {
  processVersion (ptc);
  showAddress (ptc);
  showMac (ptc);
  //processRouteCnt (ptc);
  processConfiguration (ptc);
  processTemp (ptc);
  processAdc (ptc);
}

void processEcho (pTCONN ptc) {
    char *ptr = skipWord (ptc->cmdStr);
    if (*ptr) {
	put_str(ptc, ptr);
	put_crlf(ptc);
    } else {
	put_str(ptc, "No text parameter\n");
    }
}

void processMsec (pTCONN ptc) {
    //char *ptr = skipWord (ptc->cmdStr);
    uint8_t hr, min, sec;
    uint32_t msec = srmMsecs;
    srmMsecsCurr = msec;
    hr = (uint8_t)(msec/(1000UL * 3600UL));
    min = (uint8_t)((msec%(1000UL * 3600UL)) / (1000UL * 60UL));
    sec = (uint8_t)((msec%(1000UL * 60UL)) / 1000UL);
    msec %= 1000UL;
    //sprintf (outStr, "%u:%02d:%02d.%03d, msec: 0x%X, diff: %u\n"
    //	,hr, min, sec, (uint16_t)msec
    //	, srmMsecsCurr, srmMsecsCurr - srmMsecsLast);
    ltoa (srmMsecs, ptc->outStr, 10);
    strcat (ptc->outStr, "\n");
    put_outstr(ptc);
    srmMsecsLast = srmMsecsCurr;
}

//void processTemp (pTCONN ptc) {
//	getTempValue();
//	lTemp = (int32_t)sTemp;
//	sprintf(outStr, "Temp: %d.%02d C\n", (int)lTemp/4, ((int)lTemp % 4) * 25);
//	put_outstr(ptc);
//}
//
//void processAdc (pTCONN ptc) {
//	getVccValue();
//	lAdc = (int32_t)sAdc;
//	lAdc *= 3 * 120;
//	lAdc /= 1024;
//	sprintf(outStr, "Vdd: %d.%02d V\n", lAdc/100, lAdc%100);
//	put_outstr(ptc);
//}

void processMemRead(pTCONN ptc) {
  char *ptr = skipWord (ptc->cmdStr);
  int addr = 0, cnt = 8;
  if (*ptr != 0) {
    addr = seatoi(ptr);
    ptr = skipWord (ptr);
    if (*ptr != 0)
      cnt = seatoi(ptr);
  }
  while (cnt) {
    char k = 8;
    if (cnt < 8)
      k = cnt;
    cnt -= k;
    put_str (ptc, itoa(addr, ptc->outStr, 16));
    put_ch (ptc, ':');
    while (k--) {
      put_ch (ptc, ' ');
      put_ch (ptc, hexTab[((*(uint8_t*)addr) >> 4) & 0xf]);
      put_ch (ptc, hexTab[((*(uint8_t*)addr) >> 0) & 0xf]);
    }
  put_ch(ptc, '\n');
  }
}

void processPause (pTCONN ptc) {
  uint32_t pause, msec;
  char *ptr = skipWord (ptc->cmdStr);
  if (*ptr != 0) {
    pause = seatoi(ptr);
    msec = srmMsecs;
    put_str (ptc, "Pause start: ");
    put_str (ptc, ltoa (pause, ptc->outStr, 10));
    while (srmMsecs - msec < pause) {
      //uart_process();
    }
    put_str (ptc, ", Pause end\n");
  } else {
  }
}

#ifdef SE_EEPROM
char xferSpim(uint8_t const *tx, uint8_t *rx, uint8_t len);
uint8_t eeBuffer[10];
uint8_t eeAddress, eeCount;
#define EE_CMD_WRSR   0x01
#define EE_CMD_WRITE  0x02
#define EE_CMD_READ   0x03
#define EE_CMD_WRDI   0x04
#define EE_CMD_RDSR   0x05
#define EE_CMD_WREN   0x06

void processEeRead(pTCONN ptc) {
  char *ptr = skipWord (ptc->cmdStr);
  eeCount = 8;
  if (*ptr == 0) 
    return;
  eeAddress = seatoi(ptr);
  ptr = skipWord (ptr);
  if (*ptr != 0)
    eeCount = seatoi(ptr);
  while (eeCount) {
    uint8_t k, cnt;
    if (eeCount < 8)
      cnt = eeCount;
    else
      cnt = 8;
    eeBuffer[0] = EE_CMD_READ;
    eeBuffer[1] = eeAddress;
    xferSpim((uint8_t const *)eeBuffer, eeBuffer, cnt + 2);
    put_str (ptc, itoa(eeAddress, ptc->outStr, 16));
    put_ch (ptc, ':');
    for (k = 0; k < cnt; k++) {
      put_ch (ptc, ' ');
      put_ch (ptc, hexTab[(eeBuffer[k + 2] >> 4) & 0xf]);
      put_ch (ptc, hexTab[(eeBuffer[k + 2] >> 0) & 0xf]);
    }
    put_crlf(ptc);
    eeCount -= cnt;
    eeAddress += cnt;
  }
}

void eeSendCmd(uint8_t ena) {
  xferSpim((uint8_t const *)&ena, &ena, 1);
}

void processEeStatus(pTCONN ptc) {
  char rv = 0, *ptr = skipWord (ptc->cmdStr);
  if (*ptr != 0){
    //eeSendCmd(EE_CMD_WREN);
    eeBuffer[0] = EE_CMD_WRSR;
    eeBuffer[1] = seatoi(ptr);
    xferSpim((uint8_t const *)eeBuffer, eeBuffer, 2);
  }
  eeBuffer[0] = EE_CMD_RDSR;
  xferSpim((uint8_t const *)eeBuffer, eeBuffer, 2);
  put_str (ptc, "EE status: 0x");
  put_str (ptc, itoa(eeBuffer[1], ptc->outStr, 16));
  put_crlf(ptc);
}

void processEeWrite(pTCONN ptc){
  uint8_t cnt;
  char *ptr = skipWord (ptc->cmdStr);
  if (*ptr == 0)
    return;
  eeAddress = seatoi(ptr);
  eeCount = 0;
  ptr = skipWord (ptr);
  while (*ptr != 0) {
    eeSendCmd(EE_CMD_WREN);
    eeBuffer[0] = EE_CMD_WRITE;
    eeBuffer[1] = eeAddress++;
    eeBuffer[2] = seatoi(ptr);
    xferSpim((uint8_t const *)eeBuffer, eeBuffer, 3);
    eeCount ++;
    ptr = skipWord (ptr);
    cnt = 0;
    while (++cnt < 250) {
      eeBuffer[0] = EE_CMD_RDSR;
      xferSpim((uint8_t const *)eeBuffer, eeBuffer, 2);
      if ((eeBuffer[1] & 1) == 0)
        break;
    }
  }
  eeSendCmd(EE_CMD_WRDI);
  put_str (ptc, "SPI wrote: ");
  put_str (ptc, itoa(eeCount, ptc->outStr, 10));
  put_str (ptc, ", wait cnt: ");
  put_str (ptc, itoa(cnt, ptc->outStr, 10));
  put_crlf(ptc);
}
#endif //ifdef SE_EEPROM

void processHelp (pTCONN ptc) {
	char found = 0, n, clen, k, *ptr = skipWord (ptc->cmdStr);
	char len = (char)strlen ((char *)ptr);
	for (k = 0; k < tCmdsSize; k++) {
		if (len == 0 ||strncasecmp ((char *)ptr, tCmds[k].cmd, (int)len) == 0) {
			found ++;
			clen = (char)strlen (tCmds[k].cmd);
			for (n = 0; n < clen; n++) {
				put_ch(ptc, (n < tCmds[k].cmdLen) ? toupper(tCmds[k].cmd[n]) : tolower(tCmds[k].cmd[n]));
			}
			put_str(ptc, " - ");
			clen = strlen (tCmds[k].help);
			if (clen)
				put_str(ptc, tCmds[k].help);
			put_crlf(ptc);
		}
	}
	if (found == 0) {
		put_str(ptc, "Help cmd not found: ");
		put_str(ptc, ptr);
		put_crlf(ptc);
	}
}

void processCommandTcp (pTCONN ptc) {
  int16_t k, len = 0;
  char *cmd = ptc->cmdStr;
  if (*cmd == '@') {
    quietFlag = 1;
    cmd ++;
  }
  if (*cmd == '#') {
    noTimeFlag = 1;
    cmd ++;
  }
  while (cmd[len] != 0 && cmd[len] != ' ')
    len ++;
  for (k = 0; k < tCmdsSize; k++) {
    if (len >= tCmds[k].cmdLen && strncasecmp (cmd, tCmds[k].cmd, len) == 0) {
      if (tCmds[k].function != 0)
        tCmds[k].function (ptc);
      break;
    } //if
  } //for
  noTimeFlag = 0;
  if (k == tCmdsSize) {
    put_str(ptc, "Cmd not found: ");
    put_str(ptc, cmd);
    put_crlf(ptc);
  }
  ptc->cmdPtr = 0;
  ptc->cmdStr[0] = 0;
}

void expandShortCommand (pTCONN ptc) {
  int16_t k;
  if (strchr (ptc->cmdStr, ' ') != 0) //space entered, do not expand
    return;
  for (k = 0; k < tCmdsSize; k++) {
    if (strncasecmp (ptc->cmdStr, tCmds[k].cmd, ptc->cmdPtr) == 0) {
      return; //command found, nothing to do
    } //if
  } //for
  for (k = 0; k < tCmdsSize; k++) {
    if (ptc->cmdPtr > tCmds[k].cmdLen 
      && strncasecmp (ptc->cmdStr, tCmds[k].cmd, ptc->cmdPtr) == 0) {
      //we are already on new line, so do not show command name
      strcpy (ptc->cmdStr, tCmds[k].cmd);
      strcat (ptc->cmdStr, " ");
      ptc->cmdPtr = strlen (ptc->cmdStr);
      break;
    }
  }
}

/*
1b 5b 41 up
1b 5b 42 down
1b 5b 43 right
1b 5b 44 left
1b 5b 31 7e home
1b 5b 32 7e ins
1b 5b 34 7e end
1b 5b 35 7e pgup
1b 5b 36 7e pgdn
7f delete
1b 4f 50 F1
*/

#ifdef PROCESS_ESCAPES
/*
enum _cursor_mode{
	 X_CURSOR_UP
	,X_CURSOR_DOWN
	,X_CURSOR_RIGHT
	,X_CURSOR_LEFT
	,X_CURSOR_HOME
	,X_CURSOR_INS
	,X_CURSOR_END
	,X_CURSOR_PGUP
	,X_CURSOR_PGDN
};
*/
#endif //ifdef PROCESS_ESCAPES

void clearLineTcp (pTCONN ptc) {
	while (ptc->cmdPtr > 0) {
		put_str(ptc, "\b \b");
		ptc->cmdStr[--ptc->cmdPtr] = 0;
	}
}

void processTab (pTCONN ptc) {
	//int16_t index, k, len = PconstPT_LEN + ptc->cmdPtr;
	int16_t index, k, len = ptc->cmdPtr;
	int16_t chars = 8 - (len & 7);
	while (chars --)
		put_ch(ptc, '\b');
	if (ptc->cmdPtr == 0)
		return;
	if (ptc->tabCmdSel == -1 && strchr (ptc->cmdStr, ' ')) //we are beyond command, no command expansion
		k = tCmdsSize;
	else {
		if (ptc->tabCmdSel == -1) {
			ptc->tabCmdSel = 0;
			ptc->tabPosition = ptc->cmdPtr;
		}
		for (k = 0; k < tCmdsSize; k++) {
			index = ptc->tabCmdSel + k;
			if (index >= tCmdsSize)
				index -= tCmdsSize;
			if (strncasecmp (ptc->cmdStr, tCmds[index].cmd, ptc->tabPosition) == 0) {
				while (ptc->cmdPtr --)
					put_str(ptc, "\b \b");
				put_str(ptc, tCmds[index].cmd);
				put_ch(ptc, ' ');
				strcpy (ptc->cmdStr, tCmds[index].cmd);
				strcat (ptc->cmdStr, " ");
				ptc->cmdPtr = strlen (ptc->cmdStr);
				ptc->tabCmdSel = index + 1; //next command to check for next tab
				if (ptc->tabCmdSel == tCmdsSize)
					ptc->tabCmdSel = 0;
				break;
			}
		}
	}
	if (k == tCmdsSize) {
		strcat (ptc->cmdStr, " ");
		ptc->cmdPtr ++;
		put_ch(ptc, ' ');
	}
}

uint8_t debugCount;
void debugTrap(void) {
	debugCount ++;
}


void processInput (pTCONN ptc, uint8_t i) {
  if (debugFlag & 0x80) {
    put_ch(ptc, '[');
    put_ch(ptc, hexTab[(i >> 4) & 0xf]);
    put_ch(ptc, hexTab[(i >> 0) & 0xf]);
    put_ch(ptc, ']');
  }
  if (i != '\t')
    ptc->tabCmdSel = -1;
#ifdef PROCESS_ESCAPES
  if (escapeFlag == 1) {
    if (i == 0x5b) {
      escapeFlag = 2;
      return;
    } else {
      clearLineTcp (ptc);
      escapeFlag = 0;
    }
  }
  if (escapeFlag == 2) {
    switch (i) {
      default: escapeFlag = 0; break;
    }
  }
#endif //ifdef PROCESS_ESCAPES
  switch(i) {
  case 3:
    //put_str(ptc, "\b \b"); //erase Ctrl-C from screen
    //processDisconnect (ptc);
  break;
  case 0x1b:
  //put_str(ptc, "\b \b"); //erase escape
#ifdef PROCESS_ESCAPES
    if (escapeFlag == 0)
      escapeFlag = 1;
    else
#endif //ifdef PROCESS_ESCAPES
      clearLineTcp (ptc);
  break;
  case '\t': processTab (ptc); break;
  case 0x7f:
  case '\b':
    if (ptc->cmdPtr > 0) {
      put_str(ptc, "\b \b");
      ptc->cmdStr[--ptc->cmdPtr] = 0;
    //} else {
    //	put_ch(ptc, ' \b'); //move cursor up
    }
  break;
  case '\r'://	break;	//ignore CR
  case '\n':
    if (ptc->cmdPtr > 0) {
      if (echoInput)
        put_ch(ptc, '\n');
      debugFlag &= ~0x10;
      expandShortCommand (ptc);
      processCommandTcp (ptc);
      quietFlag = 0;
    }
  break;
  default:
    if (i >= ' ' && ptc->cmdPtr < MAX_CMD_LEN - 1) {
      ptc->cmdStr[ptc->cmdPtr++] = i;
      ptc->cmdStr[ptc->cmdPtr] = 0;
      if (echoInput)
        put_ch(ptc, i);
    } else if (i < ' ') {
      if (echoInput) {
        put_ch(ptc, '<');
        put_ch(ptc, hexTab[(i >> 4) & 0xf]);
        put_ch(ptc, hexTab[(i >> 0) & 0xf]);
        put_ch(ptc, '>');
      }
      if (ptc->cmdPtr > 0)
        put_str(ptc, ptc->cmdStr);
    }
  break;
  }//switch
}

int8_t processDeviceInput (pTCONN ptc, uint8_t i) {
    ptc->retVal = 0;
    processInput (ptc, i);
    return ptc->retVal;
}

void processUartInput (uint8_t cr) {processDeviceInput (&uartTc, cr);}
void processUartFlush (void) {put_flush(&uartTc);}

/*
int8_t showPromptWTest(pTCONN ptc) {
	if(put_get_space(ptc) < strlen((char *)strCommandPrompt) + 4)
		return -1;
	put_str(ptc, (char *)strCommandPrompt);
	return 0;
}
*/
/*
int16_t sendSpiData (int8_t *buffer, int16_t cnt) {
	if (buffer != 0) {
		SPIBUF = buffer[cnt];
		SPIBUF = buffer[cnt + 1];
	} else { //direct transfer from HPI into SD card
		uint16_t val = getHpiData();
		SPIBUF = (uint8_t)(val & 0xff);
		SPIBUF = (uint8_t)(val >> 8);
	}
	return cnt + 2;
}

void sendCrcData (void) {
	SPIBUF = 0xff;
}

uint8_t getSpiData(void) {
	uint8_t data_token = SPIBUF;
	return data_token;
}

int16_t getSpiTxSize (void) {
	int16_t txSz = (SPISTAT >> 16) & 0x1F;
	return txSz;
}

int16_t getSpiRxSize (void) {
	int16_t rxSz = (SPISTAT >> 24) & 0x1F;
	return rxSz;
}
*/

void put_flush(pTCONN ptc) {
  if (ptc->stdoutPtr) {
    nrf_mesh_serial_tx(ptc->stdoutBuf, ptc->stdoutPtr);
    ptc->stdoutPtr = 0;
  }
}

char *put_array(pTCONN ptc, char *data, unsigned len) {
  if (len + ptc->stdoutPtr > ARR_SIZE(ptc->stdoutBuf)) {
    put_flush(ptc);
  }
  if (len >= ARR_SIZE(ptc->stdoutBuf)) {
    nrf_mesh_serial_tx((uint8_t*)data, len);
    return data;
  } 
  memcpy(ptc->stdoutBuf + ptc->stdoutPtr, data, len);
  ptc->stdoutPtr += len;
  if (ptc->stdoutPtr == ARR_SIZE(ptc->stdoutBuf)) {
    put_flush(ptc);
  }
  return data;
}

char *put_str(pTCONN ptc, char *data) {
  return put_array(ptc, data, strlen(data));
}

char *put_outstr(pTCONN ptc) {
  return put_str(ptc, ptc->outStr);
}

void put_crlf(pTCONN ptc) {
  put_str(ptc, "\n");
}

bool put_ch(pTCONN ptc, char ch) {
  ptc->stdoutBuf[ptc->stdoutPtr++] = ch;
  if (ptc->stdoutPtr == ARR_SIZE(ptc->stdoutBuf)) {
    put_flush(ptc);
  }
  return true;
}

void showByteHex(pTCONN ptc, uint8_t val) {
    put_ch(ptc, hexTab[(val >> 4) & 0xf]);
    put_ch(ptc, hexTab[(val >> 0) & 0xf]);
}

void showByteDec(pTCONN ptc, uint8_t val) {
  if (val >= 100) {
    put_ch(ptc, '0' + (val / 100));
    val %= 100;
    put_ch(ptc, '0' + (val / 10));
  } else if (val >= 10) {
    put_ch(ptc, '0' + (val / 10));
  }
  put_ch(ptc, '0' + (val % 10));
}

char valueStr[10];
char valueStrLen = 0;
void showWordDec(pTCONN ptc, uint16_t val) {
	valueStrLen = 0;
	valueStr[5] = 0;
	valueStr[4] = '0' + val % 10;
	val /= 10;
	while (val) {
		valueStrLen++;
		valueStr[4-valueStrLen] = '0' + val % 10;
		val /= 10;
	}
	put_str(ptc, valueStr + (4 - valueStrLen));
}

void showDWordDec(pTCONN ptc, uint32_t val) {
	valueStrLen = 0;
	valueStr[9] = 0;
	valueStr[8] = '0' + val % 10;
	val /= 10;
	while (val) {
		valueStrLen++;
		valueStr[8-valueStrLen] = '0' + val % 10;
		val /= 10;
	}
	put_str(ptc, valueStr + (8 - valueStrLen));
}

void showDWordHex(pTCONN ptc, uint32_t val) {
	valueStrLen = 0;
	valueStr[8] = 0;
	valueStr[7] = hexTab[val & 0xf];
	val >>= 4;
	while (val) {
		valueStrLen++;
		valueStr[7-valueStrLen] = hexTab[val & 0xf];
		val >>= 4;
	}
	put_str(ptc, valueStr + (7 - valueStrLen));
}

void showWordHex(pTCONN ptc, uint16_t val) {
	valueStrLen = 0;
	valueStr[8] = 0;
	valueStr[7] = hexTab[val & 0xf];
	val >>= 4;
	while (val) {
		valueStrLen++;
		valueStr[7-valueStrLen] = hexTab[val & 0xf];
		val >>= 4;
	}
	put_str(ptc, valueStr + (7 - valueStrLen));
}

/*
TO DO: allow longer command + description then 0x40 characters
Now it will stop on longer one
*/
TCMDS tCmds[] = {
//	 {"adc", 3, "Measure Vdd", processAdc}
	 {"address", 1, "Show NW address", processAddress}
	//,{"baud", 2, "[idx] - set/get UART baudrate", processBaud}
	,{"cls", 3, "clear screen", processCls}
	//,{"configuration", 3, "show radio configuration", processConfiguration}
	//,{"datarate", 5, "[0..2] Set data rate (250k, 1M, 2Mb/s)", processDataRate}
	//,{"delay", 1, "[delay] Enter delay in msecs (1..250)", processDelay}
	,{"debug", 3, "flag - set debug flags 1:keys, 2:esc, 4:hist", processDebug}
	//,{"direct", 3, "mac text - send text string over radio", processRadioDirect}
	,{"echo", 2, "[text] - Send back text string", processEcho}
	//,{"eedisa", 3, "EEPconst write disable", processEeDisable}
	//,{"eeena", 3, "EEPconst write enable", processEeEnable}
#ifdef SE_EEPROM
	,{"eeread", 3, "[addr[cnt]] - read EE [dflt 8]", processEeRead}
	,{"eestatus", 3, "[WRSR] read/write EEstatus", processEeStatus}
	,{"eewrite", 3, "addr val [val ...] - write EE at address", processEeWrite}
#endif //ifdef SE_EEPROM
	//,{"endch", 2, "[ch] Set end channel (0..80)", processEndChannel}
	//,{"getchannel", 4, "addr - get active radio channel for device", processGetChannel}
	,{"help", 1, "Show commands help", processHelp}
	,{"led", 1, "ena|dis|[x [val]] - set/get led status", processLed}
	//,{"logrt", 3, "[clr] - clear/get R/T log", processLogRt}
	,{"mac", 3, "[mac] Set/get element unique MAC", processMac}
	,{"memread", 4, "[addr [cnt]] - read memory (continue) [dflt 8]", processMemRead}
	,{"msec", 2, "[mode] - get msec timer", processMsec}
	,{"nwid", 2, "[id] Set/get network ID", processNwid}
#ifdef SUPPORT_DEVICE_TESTS		
	,{"oper", 2, "[0..5] Set test radio operation", processRadioTest}
#endif //ifdef SUPPORT_DEVICE_TESTS		
	,{"pause", 2, "msec - Pause execution for mssecs", processPause}
	//,{"power", 3, "[0..7] Set TX power (0=+4 dBm,..,7=-30 dBm)", processPower}
	//,{"pic", 2, "get PIC EEPconst data", processGetPic}
	//,{"print", 3, "text - print text to stdout", processPrint}
	//,{"quit", 1, "Exit session", processDisconnect}
	//,{"radio", 2, "dst text - send text string over radio", processRadioText}
	//,{"reset", 3, "restart CPU", processReset}
	,{"revision", 3, "show app version", processRevision}
	//,{"rng", 3, "get random number", processRng}
	//,{"rxdelay", 3, "[delay] - Set/get interpacket delay in us", processRadioDelay}
	//,{"temp", 3, "Get board temperature", processTemp}
	//,{"test", 3, "cnt addr power - run direct connection test", processTest}
	//,{"trace", 2, "dest level - trace route - answer after specific number of hops", processTrace}
	//,{"tx", 2, "dst ptyp payload - send test packet", processRadioTx}
	,{"type", 2, "[type|?] Set/get module type (0-elu, 1-gw, 2-rel)", processType}
	//,{"uart", 2, "[baud [tx rx]] - configure UART", processUartConfig}
	//,{"ucls", 2, "Clear screen on uart console", processUartCls}
	,{"version", 1, "show build version", processVersion}
#ifdef USE_WATCHDOG
	,{"wdt", 2, "[1|2|3|4] - test watchdog, 1-temp, 3-test pin on, 4-test off", processWatchdog}
#endif //ifdef USE_WATCHDOG
};
uint8_t tCmdsSize = ARR_SIZE(tCmds);

